@@ -129,6 +129,7 @@ var parsers = new FreeList('parsers', 1000, function() {
     }
     if (!parser.incoming.upgrade) {
       // For upgraded connections, also emit this after parser.execute
+      parser.incoming.readable = false;
       parser.incoming.emit('end');
     }
   };
@@ -975,12 +976,20 @@ function connectionListener(socket) {
   var self = this;
   var outgoing = [];
   var incoming = [];
+  var abortError = null;
 
   function abortIncoming() {
+    if (!abortError) {
+      abortError = new Error('http.ServerRequest was aborted by the client');
+      abortError.code = 'aborted';
+    }
+
     while (incoming.length) {
       var req = incoming.shift();
+
+      // @deprecated, should be removed in 0.5.x
       req.emit('aborted');
-      req.emit('close');
+      req.emit('close', abortError);
     }
     // abort socket._httpMessage ?
   }
@@ -991,6 +1000,8 @@ function connectionListener(socket) {
 
   socket.setTimeout(2 * 60 * 1000); // 2 minute timeout
   socket.addListener('timeout', function() {
+    abortError = new Error('http.ServerRequest timed out');
+    abortError.code = 'timeout';
     socket.destroy();
   });
 
@@ -1078,7 +1089,11 @@ function connectionListener(socket) {
     // When we're finished writing the response, check if this is the last
     // respose, if so destroy the socket.
     res.on('finish', function() {
-      assert(incoming[0] === req);
+      // Usually the first incoming element should be our request.  it may
+      // be that in the case abortIncoming() was called that the incoming
+      // array will be empty.
+      assert(incoming.length == 0 || incoming[0] === req);
+
       incoming.shift();
 
       res.detachSocket(socket);
@@ -1185,7 +1200,7 @@ Agent.prototype._establishNewConnection = function() {
   parser.incoming = null;
 
   socket.on('error', function(err) {
-    debug('AGENT SOCKET ERROR: ' + err.message);
+    debug('AGENT SOCKET ERROR: ' + err.message + '\n' + err.stack);
     var req;
     if (socket._httpMessage) {
       req = socket._httpMessage;
@@ -1193,13 +1208,12 @@ Agent.prototype._establishNewConnection = function() {
       req = self.queue.shift();
       assert(req._queue === self.queue);
       req._queue = null;
-    } else {
-      // No requests on queue? Where is the request
-      assert(0);
     }
 
-    req.emit('error', err);
-    req._hadError = true; // hacky
+    if (req) {
+      req.emit('error', err);
+      req._hadError = true; // hacky
+    }
 
     // clean up so that agent can handle new requests
     parser.finish();
