@@ -45,23 +45,66 @@ Stream.prototype.pipe = function(dest, options) {
 
   dest.on('drain', ondrain);
 
-  /*
-   * If the 'end' option is not supplied, dest.end() will be called when
-   * source gets the 'end' event.
-   */
-
+  // If the 'end' option is not supplied, dest.end() will be called when
+  // source gets the 'end' or 'close' events.  Only dest.end() once, and
+  // only when all sources have ended.
   if (!options || options.end !== false) {
-    function onend() {
-      dest.end();
-    }
+    dest._pipeCount = dest._pipeCount || 0;
+    dest._pipeCount++;
 
     source.on('end', onend);
+    source.on('close', onclose);
   }
 
-  /*
-   * Questionable:
-   */
+  var didOnEnd = false;
+  function onend() {
+    if (didOnEnd) return;
+    didOnEnd = true;
 
+    dest._pipeCount--;
+
+    // remove the listeners
+    cleanup();
+
+    if (dest._pipeCount > 0) {
+      // waiting for other incoming streams to end.
+      return;
+    }
+
+    dest.end();
+  }
+
+
+  function onclose() {
+    if (didOnEnd) return;
+    didOnEnd = true;
+
+    dest._pipeCount--;
+
+    // remove the listeners
+    cleanup();
+
+    if (dest._pipeCount > 0) {
+      // waiting for other incoming streams to end.
+      return;
+    }
+
+    dest.destroy();
+  }
+
+  // don't leave dangling pipes when there are errors.
+  function onerror(er) {
+    cleanup();
+    if (this.listeners('error').length === 1) {
+      throw er; // Unhandled stream error in pipe.
+    }
+  }
+
+  source.on('error', onerror);
+  dest.on('error', onerror);
+
+  // guarantee that source streams can be paused and resumed, even
+  // if the only effect is to proxy the event back up the pipe chain.
   if (!source.pause) {
     source.pause = function() {
       source.emit('pause');
@@ -73,34 +116,40 @@ Stream.prototype.pipe = function(dest, options) {
       source.emit('resume');
     };
   }
-  
-  var onpause = function() {
+
+  function onpause() {
     source.pause();
   }
 
   dest.on('pause', onpause);
-  
-  var onresume = function() {
+
+  function onresume() {
     if (source.readable) source.resume();
-  };
-  
+  }
+
   dest.on('resume', onresume);
-  
-  var cleanup = function () {
+
+  // remove all the event listeners that were added.
+  function cleanup() {
     source.removeListener('data', ondata);
     dest.removeListener('drain', ondrain);
+
     source.removeListener('end', onend);
-    
+    source.removeListener('close', onclose);
+
     dest.removeListener('pause', onpause);
     dest.removeListener('resume', onresume);
-    
+
+    source.removeListener('error', onerror);
+    dest.removeListener('error', onerror);
+
     source.removeListener('end', cleanup);
     source.removeListener('close', cleanup);
-    
+
     dest.removeListener('end', cleanup);
     dest.removeListener('close', cleanup);
   }
-  
+
   source.on('end', cleanup);
   source.on('close', cleanup);
 
